package com.landray.kmss.sys.attachment.jg;

import com.landray.kmss.sys.attachment.integrate.wps.util.SysAttWpsCloudUtil;
import com.landray.kmss.sys.attachment.util.SysAttConfigUtil;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.engine.jdbc.NonContextualLobCreator;
import com.landray.kmss.util.FileMimeTypeUtil;
import java.net.URLDecoder;
import com.landray.kmss.util.StringUtil;
import com.landray.kmss.sys.organization.model.SysOrgPerson;
import com.landray.kmss.util.SpringBeanUtil;
import com.landray.kmss.sys.organization.service.ISysOrgPersonService;
import java.net.URLEncoder;
import com.landray.kmss.common.model.IBaseModel;
import java.io.File;
import com.landray.kmss.util.IDGenerator;
import com.landray.kmss.util.FileUtil;
import com.landray.kmss.common.actions.RequestContext;
import com.landray.kmss.util.DateUtil;
import java.util.Date;
import com.landray.kmss.util.UserUtil;
import com.landray.kmss.util.ResourceUtil;
import DBstep.iMsgServer2000;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

public class SysAttJGSaveFileFunction extends AbstractSysAttachmentJGFunction
{
    private static final Logger logger;
    
    static {
        logger = LoggerFactory.getLogger(SysAttJGSaveFileFunction.class);
    }
    
    private void loggerInfo(final SysAttMain sysAttMain, final iMsgServer2000 MsgObj) {
        ResourceUtil.getKmssConfigString("kmss.isLogJgEnabled");
        if (SysAttJGSaveFileFunction.logger.isInfoEnabled()) {
            MsgObj.GetMsgByName("RECORDID");
            MsgObj.GetMsgByName("USERNAME");
            MsgObj.GetMsgByName("FILENAME");
            MsgObj.GetMsgByName("FILETYPE");
            final int mFileSize = MsgObj.MsgFileSize();
            SysAttJGSaveFileFunction.logger.info("iWeboffice控件监控日志:1、【主文档ModelId】：" + MsgObj.GetMsgByName("_fdModelId") + "；" + "2、【主文档ModelName】：" + MsgObj.GetMsgByName("_fdModelName") + "；" + "3、【附件fdId】：" + sysAttMain.getFdId() + "；" + "4、【文档名称】：" + sysAttMain.getFdFileName() + "；" + "5、【文档大小】：" + mFileSize + "字节；" + "6、【用户名】：" + UserUtil.getUser().getFdName() + "；" + "7、【保存时间】：" + DateUtil.convertDateToString(new Date(), "datetime", UserUtil.getKMSSUser().getLocale()));
        }
    }
    
    @Override
    public void execute(final RequestContext request, final iMsgServer2000 MsgObj) throws Exception {
        final String mCommand = MsgObj.GetMsgByName("COMMAND");
        final String fileType = MsgObj.GetMsgByName("FILETYPE");
        Boolean canSave = false;
        if (".doc".endsWith(fileType.toLowerCase()) || ".docx".endsWith(fileType.toLowerCase()) || ".xlsx".endsWith(fileType.toLowerCase()) || ".wps".endsWith(fileType.toLowerCase()) || ".et".endsWith(fileType.toLowerCase())) {
            String fdSystemTempPath = FileUtil.getSystemTempPath();
            if (!fdSystemTempPath.endsWith("/")) {
                fdSystemTempPath = String.valueOf(fdSystemTempPath) + "/";
            }
            String tmpId = IDGenerator.generateID();
            String filePath;
            File tmpFile;
            for (filePath = String.valueOf(fdSystemTempPath) + tmpId + "_tmp" + fileType, tmpFile = new File(filePath); tmpFile.exists(); tmpFile = new File(filePath)) {
                tmpId = IDGenerator.generateID();
                filePath = String.valueOf(fdSystemTempPath) + tmpId + "_tmp" + fileType;
            }
            final File tmpPfile = tmpFile.getParentFile();
            if (!tmpPfile.exists()) {
                tmpPfile.mkdirs();
            }
            tmpFile.createNewFile();
            if (MsgObj.MsgFileSave(filePath)) {
                final long startTime = System.currentTimeMillis();
                final String ret = MsgObj.CheckFileValid(filePath);
                final long endTime = System.currentTimeMillis();
                SysAttJGSaveFileFunction.logger.info("poi程序运行时间： " + (endTime - startTime) + "ms");
                if ("ok".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("POI检验文档成功!");
                    canSave = true;
                }
                else if ("NoSheetPages".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("Sheet页为0!");
                }
                else if ("FileEncryption".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("加密文档解析异常!");
                }
                else if ("FileIsDamaged".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("文件损坏!");
                }
                else if ("EmptyDocument".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("空文档!保存");
                    canSave = true;
                }
                else if ("DocumentNotWord".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("文档类型不是word类型!");
                }
                else if ("DocumentNotExcel".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("文档类型不是excel类型!");
                }
                else if ("FileEmpty".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("文件流为空!");
                }
                else if ("TheFileFormatIsNotSupported".equals(ret)) {
                    SysAttJGSaveFileFunction.logger.info("文件类型不支持!");
                }
                else {
                    SysAttJGSaveFileFunction.logger.info("POI检验文档因未知原因失败!");
                }
                tmpFile.delete();
            }
        }
        else {
            canSave = true;
        }
        if (canSave) {
            if ("ORIGINALDRAFT".equals(mCommand) || "REVISIONDRAFT".equals(mCommand) || "CLEARDRAFT".equals(mCommand)) {
                this.doSaveDraft(request, MsgObj);
                SysAttJGSaveFileFunction.logger.info("底稿保存");
            }
            else {
                this.doSaveFile(request, MsgObj);
                SysAttJGSaveFileFunction.logger.info("正文保存");
            }
        }
    }
    
    private void doSaveDraft(final RequestContext request, final iMsgServer2000 MsgObj) throws Exception {
        MsgObj.Charset = "GB2312";
        final SysAttMain sysAttMain = new SysAttMain();
        final String fdId = MsgObj.GetMsgByName("_fdId");
        final String fdMmodelName = MsgObj.GetMsgByName("_modelName");
        final String key = MsgObj.GetMsgByName("_key");
        sysAttMain.setFdId(fdId);
        sysAttMain.setFdKey(key);
        sysAttMain.setFdModelName(fdMmodelName);
        this.setSysAttMainNew(sysAttMain, MsgObj, key);
        this.getSysAttMainService().add((IBaseModel)sysAttMain);
        this.loggerInfo(sysAttMain, MsgObj);
        if (SysAttJGSaveFileFunction.logger.isDebugEnabled()) {
            SysAttJGSaveFileFunction.logger.debug("保存附件：fdId:" + sysAttMain.getFdId() + "; fdModelId:" + MsgObj.GetMsgByName("_fdModelId") + "; fdModelName:" + MsgObj.GetMsgByName("_fdModelName") + "; fdKey:" + MsgObj.GetMsgByName("_fdKey"));
        }
        MsgObj.SetMsgByName("STATUS", "保存成功!");
        MsgObj.SetMsgByName("COMMAND", "NODRAFT");
        MsgObj.SetMsgByName("fd_fileId", sysAttMain.getFdId());
        final String fileName = sysAttMain.getFdFileName().replaceAll(" ", "%20");
        MsgObj.SetMsgByName("fd_fileName", URLEncoder.encode(fileName, "UTF-8"));
        MsgObj.MsgError("");
        MsgObj.MsgFileClear();
    }
    
    private void setSysAttMainNew(final SysAttMain sysAttMain, final iMsgServer2000 MsgObj, final String key) throws Exception {
        final String fileType = MsgObj.GetMsgByName("FILETYPE");
        String _fdFileName = MsgObj.GetMsgByName("_fdFileName");
        final String userId = MsgObj.GetMsgByName("_userId");
        MsgObj.GetMsgByName("_userName");
        SysOrgPerson p = UserUtil.getUser();
        if (p.isAnonymous()) {
            final ISysOrgPersonService sysOrgPersonService = (ISysOrgPersonService)SpringBeanUtil.getBean("sysOrgPersonService");
            p = (SysOrgPerson)sysOrgPersonService.findByPrimaryKey(userId);
        }
        if (p != null) {
            sysAttMain.setFdCreatorId(p.getFdId());
        }
        if (StringUtil.isNull(_fdFileName)) {
            final StringBuffer fileName = new StringBuffer();
            if ("originalAtt".equals(key)) {
                fileName.append(ResourceUtil.getString("sysAttMain.originalDraft", "sys-attachment"));
            }
            if ("revisionAtt".equals(key)) {
                fileName.append(ResourceUtil.getString("sysAttMain.revisionDraft", "sys-attachment"));
            }
            if ("clearAtt".equals(key)) {
                fileName.append(ResourceUtil.getString("sysAttMain.clearDraft", "sys-attachment"));
            }
            if ("redheadAtt".equals(key)) {
                fileName.append(ResourceUtil.getString("sysAttMain.redheadDraft", "sys-attachment"));
            }
            fileName.append("_");
            fileName.append(p.getFdName());
            fileName.append("_");
            final String filename = MsgObj.GetMsgByName("FILENAME");
            final String mainOnlineSimpleName = filename.substring(0, filename.lastIndexOf("."));
            fileName.append(URLDecoder.decode(mainOnlineSimpleName, "UTF-8"));
            fileName.append("_");
            fileName.append(DateUtil.convertDateToString(new Date(), DateUtil.PATTERN_DATE));
            if (StringUtil.isNotNull(fileName.toString()) && StringUtil.isNotNull(fileType) && !fileName.toString().endsWith(fileType)) {
                fileName.append(fileType);
            }
            if (".do".equals(fileName.substring(fileName.length() - 3, fileName.length()).toLowerCase()) && "office".equals(MsgObj.GetMsgByName("_attType"))) {
                fileName.append("c");
            }
            _fdFileName = fileName.toString();
            _fdFileName = _fdFileName.replaceAll("[/\\\\:*?|]", "-");
            _fdFileName = _fdFileName.replaceAll("[\"<>]", "'");
        }
        else {
            _fdFileName = URLDecoder.decode(_fdFileName, "UTF-8");
        }
        sysAttMain.setFdFileName(_fdFileName);
        sysAttMain.setFdAttType(MsgObj.GetMsgByName("_attType"));
        sysAttMain.setFdContentType(FileMimeTypeUtil.getContentType(_fdFileName));
        sysAttMain.setFdSize(Double.valueOf((double)MsgObj.MsgFileSize()));
        if (sysAttMain.getDocCreateTime() == null) {
            sysAttMain.setDocCreateTime(new Date());
        }
        sysAttMain.setInputStream(NonContextualLobCreator.INSTANCE.createBlob(MsgObj.MsgFileBody()).getBinaryStream());
    }
    
    private void doSaveFile(final RequestContext request, final iMsgServer2000 MsgObj) throws Exception {
        String fdId = MsgObj.GetMsgByName("_fdId");
        String modelId = MsgObj.GetMsgByName("RECORDID");
        final String modelName = MsgObj.GetMsgByName("_fdModelName");
        final String key = MsgObj.GetMsgByName("_fdKey");
        final String extParam = MsgObj.GetMsgByName("EXTPARAM");
        final String userId = MsgObj.GetMsgByName("_userId");
        MsgObj.GetMsgByName("_userName");
        if (StringUtils.isNotBlank(extParam) && "pdf2018Signature".equals(extParam)) {
            fdId = MsgObj.GetMsgByName("RECORDID");
            modelId = MsgObj.GetMsgByName("_fdModelId");
        }
        if (SysAttJGSaveFileFunction.logger.isDebugEnabled()) {
            SysAttJGSaveFileFunction.logger.debug("保存文档：fdId:" + fdId + "; fdModelId:" + modelId + "; fdModelName:" + modelName + "; fdKey:" + key);
        }
        SysAttMain sysAttMain = this.getSysAttMain(fdId, modelId, modelName, key, true);
        SysOrgPerson p = UserUtil.getUser();
        if (p.isAnonymous()) {
            final ISysOrgPersonService sysOrgPersonService = (ISysOrgPersonService)SpringBeanUtil.getBean("sysOrgPersonService");
            p = (SysOrgPerson)sysOrgPersonService.findByPrimaryKey(userId);
        }
        if (sysAttMain == null) {
            sysAttMain = new SysAttMain();
            if (p != null) {
                sysAttMain.setFdCreatorId(p.getFdId());
            }
            this.setSysAttMain(sysAttMain, MsgObj);
            this.getSysAttMainService().add((IBaseModel)sysAttMain);
        }
        else {
            this.setSysAttMain(sysAttMain, MsgObj);
            this.getSysAttMainService().updateByUser((IBaseModel)sysAttMain, p.getFdId());
            if (("0".equals(SysAttConfigUtil.getOnlineToolType()) && "2".equals(SysAttConfigUtil.isReadJGForMobile())) || ("3".equals(SysAttConfigUtil.getOnlineToolType()) && "2".equals(SysAttConfigUtil.isReadWPSForMobile())) || ("1".equals(SysAttConfigUtil.getOnlineToolType()) && "2".equals(SysAttConfigUtil.isReadWPSCloudForMobile()) && StringUtil.isNotNull(sysAttMain.getFdKey()) && !"originalAtt".equals(sysAttMain.getFdKey()))) {
                SysAttWpsCloudUtil.syncAttToAddByMainId(sysAttMain.getFdId());
            }
        }
        this.loggerInfo(sysAttMain, MsgObj);
        MsgObj.SetMsgByName("STATUS", "保存成功!");
        MsgObj.SetMsgByName("_fdAttMainId", sysAttMain.getFdId());
        MsgObj.MsgError("");
        MsgObj.MsgFileClear();
    }
    
    private void setSysAttMain(final SysAttMain sysAttMain, final iMsgServer2000 MsgObj) throws Exception {
        final String extParam = MsgObj.GetMsgByName("EXTPARAM");
        if (StringUtils.isNotBlank(extParam) && "pdf2018Signature".equals(extParam)) {
            if (StringUtil.isNotNull(MsgObj.GetMsgByName("_fdModelId"))) {
                sysAttMain.setFdModelName(MsgObj.GetMsgByName("_fdModelId"));
            }
        }
        else if (StringUtil.isNotNull(MsgObj.GetMsgByName("RECORDID"))) {
            sysAttMain.setFdModelId(MsgObj.GetMsgByName("RECORDID"));
        }
        if (StringUtil.isNotNull(MsgObj.GetMsgByName("_fdModelName"))) {
            sysAttMain.setFdModelName(MsgObj.GetMsgByName("_fdModelName"));
        }
        if (StringUtil.isNotNull(MsgObj.GetMsgByName("_fdKey")) && sysAttMain.getFdKey() == null) {
            sysAttMain.setFdKey(MsgObj.GetMsgByName("_fdKey"));
        }
        final String fileType = MsgObj.GetMsgByName("FILETYPE");
        String fileName = MsgObj.GetMsgByName("FILENAME");
        fileName = fileName.replaceAll("%(?![0-9a-fA-F]{2})", "%25");
        fileName = fileName.replaceAll("\\+", "%2B");
        fileName = URLDecoder.decode(fileName, "UTF-8");
        if (StringUtil.isNotNull(fileName) && StringUtil.isNotNull(fileType) && !fileName.endsWith(fileType)) {
            fileName = String.valueOf(fileName) + fileType;
        }
        if (".do".equals(fileName.substring(fileName.length() - 3, fileName.length()).toLowerCase()) && "office".equals(MsgObj.GetMsgByName("_attType"))) {
            fileName = String.valueOf(fileName) + "c";
        }
        if (StringUtil.isNull(sysAttMain.getFdFileName())) {
            sysAttMain.setFdFileName(fileName);
        }
        else if ("mainOnline".equals(sysAttMain.getFdKey()) && "com.landray.kmss.km.agreement.model.KmAgreementApply".equals(sysAttMain.getFdModelName())) {
            final int idx = sysAttMain.getFdFileName().lastIndexOf(".");
            final String mainOnlineSimpleName = sysAttMain.getFdFileName().substring(0, idx);
            sysAttMain.setFdFileName(String.valueOf(mainOnlineSimpleName) + fileType);
        }
        sysAttMain.setFdAttType(MsgObj.GetMsgByName("_attType"));
        sysAttMain.setFdContentType(FileMimeTypeUtil.getContentType(fileName));
        sysAttMain.setFdSize(Double.valueOf((double)MsgObj.MsgFileSize()));
        if (sysAttMain.getDocCreateTime() == null) {
            sysAttMain.setDocCreateTime(new Date());
        }
        sysAttMain.setInputStream(NonContextualLobCreator.INSTANCE.createBlob(MsgObj.MsgFileBody()).getBinaryStream());
        if (!"true".equals(MsgObj.GetMsgByName("_saveDraft")) && (StringUtil.isNotNull(sysAttMain.getFdPersonId()) || sysAttMain.getFdLastOpenTime() != null)) {
            sysAttMain.setFdPersonId((String)null);
            sysAttMain.setFdLastOpenTime((Date)null);
        }
    }
}
